//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Job/FitSuiteItem.cpp
//! @brief     Implements class FitSuiteItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Job/FitSuiteItem.h"
#include "GUI/Model/Job/FitParameterContainerItem.h"
#include "GUI/Model/Job/MinimizerItem.h"
#include "GUI/Model/Model/SessionModel.h"
#include "GUI/Support/XML/UtilXML.h"
#include "GUI/Util/Error.h"

namespace {

namespace Tag {

const QString FitParameterContainer("FitParameterContainer");

} // namespace Tag

} // namespace

FitSuiteItem::FitSuiteItem()
    : SessionItem(M_TYPE)
{
    addProperty(P_UPDATE_INTERVAL, int(10));
    addProperty(P_ITERATION_COUNT, int(0));
    addProperty(P_CHI2, double(0.0));

    registerTag(T_MINIMIZER, 1, 1, {MinimizerContainerItem::M_TYPE});
}

FitParameterContainerItem* FitSuiteItem::createFitParametersContainer()
{
    if (m_fitContainer)
        throw Error("FitSuiteItem::createFitParametersContainer() -> Error. Attempt to create "
                    "a second FitParameterContainer.");

    m_fitContainer = std::make_unique<FitParameterContainerItem>();
    return m_fitContainer.get();
}

FitParameterContainerItem* FitSuiteItem::fitParameterContainerItem()
{
    return m_fitContainer.get();
}

MinimizerContainerItem* FitSuiteItem::createMinimizerContainer()
{
    if (getItem(T_MINIMIZER))
        throw Error("FitSuiteItem::createMinimizerContainer() -> Error. Attempt to create "
                    "a second MinimizerContainer.");

    return model()->insertItem<MinimizerContainerItem>(this, -1, T_MINIMIZER);
}

MinimizerContainerItem* FitSuiteItem::minimizerContainerItem()
{
    return item<MinimizerContainerItem>(T_MINIMIZER);
}

int FitSuiteItem::updateInterval() const
{
    return getItemValue(P_UPDATE_INTERVAL).toInt();
}

void FitSuiteItem::setUpdateInterval(const int interval)
{
    setItemValue(P_UPDATE_INTERVAL, interval);
    emit updateIntervalChanged(interval);
}

int FitSuiteItem::iterationCount() const
{
    return getItemValue(P_ITERATION_COUNT).toInt();
}

void FitSuiteItem::setIterationCount(const int count)
{
    setItemValue(P_ITERATION_COUNT, count);
    emit iterationCountChanged(count);
}

double FitSuiteItem::chi2() const
{
    return getItemValue(P_CHI2).toDouble();
}

void FitSuiteItem::setChi2(const double chi2)
{
    setItemValue(P_CHI2, chi2);
}

void FitSuiteItem::writeNonSessionItems(QXmlStreamWriter* writer) const
{
    // fit parameter container
    if (m_fitContainer) {
        writer->writeStartElement(Tag::FitParameterContainer);
        m_fitContainer->writeTo(writer);
        writer->writeEndElement();
    }
}

void FitSuiteItem::readNonSessionItems(QXmlStreamReader* reader)
{
    while (reader->readNextStartElement()) {

        // fit parameter container
        if (reader->name() == Tag::FitParameterContainer) {
            if (!m_fitContainer)
                createFitParametersContainer();
            m_fitContainer->readFrom(reader);
            GUI::Session::XML::gotoEndElementOfTag(reader, Tag::FitParameterContainer);

        } else {
            reader->skipCurrentElement();
        }
    }
}
